/** @module matches/state */
import { call, put, takeEvery, select } from 'redux-saga/effects'
import moment from 'moment'
import { SZTM_API } from '../local-constants'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  matchesSetFilter: (filter) => {
    return {
      type: 'MATCHES/SET_FILTER',
      filter
    }
  },
  matchesFilterApplied: (filteredMatches, filteredPlayers) => {
    return {
      type: 'MATCHES/FILTER_APPLIED',
      filteredMatches, 
      filteredPlayers
    }
  },
  matchesGetMatchesRequest: (data) => {
    return {
      type: 'MATCHES/GET_MATCHES_REQUEST',
      data
    }
  },
  matchesGetMatchesSuccess: (data) => {
    return {
      type: 'MATCHES/GET_MATCHES_SUCCESS',
      data
    }
  },
  matchesGetMatchesFailure: (error) => {
    return {
      type: 'MATCHES/GET_MATCHES_FAILURE',
      error
    }
  },
  matchesGetPlayersRequest: (data) => {
    return {
      type: 'MATCHES/GET_PLAYERS_REQUEST',
      data
    }
  },
  matchesGetPlayersSuccess: (data) => {
    return {
      type: 'MATCHES/GET_PLAYERS_SUCCESS',
      data
    }
  },
  matchesGetPlayersFailure: (error) => {
    return {
      type: 'MATCHES/GET_PLAYERS_FAILURE',
      error
    }
  },
}
console.log('State actions', actions)
const emptyFilter = {
  category: '',
  date: '',
  time: '',
  place: '',
  player: '',
  result: '',
  junior: '',
  paid: '',
  matchfilter: '',
}

/** state definition */
export const state = {
  matches: [],
  players: [],
  filteredMatches: [],
  filteredPlayers: [],
  filter: emptyFilter,
  stats: {},
  filteredStats: {},
  matchGetRequested: false,
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'MATCHES/SET_FILTER':
      return { ...state, filter: { ...state.filter, ...action.filter } }
    case 'MATCHES/FILTER_APPLIED':
      return { ...state, filteredMatches: action.filteredMatches, filteredPlayers: action.filteredPlayers }
    case 'MATCHES/GET_MATCHES_REQUEST':
      return { ...state, matchGetRequested: true }
    case 'MATCHES/GET_MATCHES_SUCCESS':
      return { ...state, matches: action.data, matchGetRequested: false }
    case 'MATCHES/GET_MATCHES_FAILURE':
      return { ...state, matchGetRequested: false }
    case 'MATCHES/GET_PLAYERS_REQUEST':
      return { ...state, matchGetRequested: true }
    case 'MATCHES/GET_PLAYERS_SUCCESS':
      return { ...state, players: action.data, matchGetRequested: false }
    case 'MATCHES/GET_PLAYERS_FAILURE':
      return { ...state, matchGetRequested: false }
    default:
      return state
  }
}

function * getMatches (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Get matches requested', action, token)
    const { data } = action
    
    const queryParams = Object.keys(data || {})
      .map(k => encodeURIComponent(k) + '=' + encodeURIComponent(data[k]))
      .join('&')
    const response = yield call(fetch, `${SZTM_API}/api/sztm/schedule?${queryParams}`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    })
    if (response.status != 200) {
      throw Error(yield response.json())
    } else {
      const responseJson = yield response.json()
      console.log(responseJson)
      yield put(actions.matchesGetMatchesSuccess(responseJson.matches))
      yield put(actions.matchesSetFilter({}))
    }
  } catch (error) {
    console.log('Matches failure!', actions.matchesGetMatchesFailure(error))
    yield put(actions.matchesGetMatchesFailure(error))
  }
}

function * getPlayers (action) {
  try {
    const token = localStorage.getItem('accessToken')
    console.log('Get players requested', action, token)
    const { data } = action
    
    const queryParams = Object.keys(data || {})
      .map(k => encodeURIComponent(k) + '=' + encodeURIComponent(data[k]))
      .join('&')
    const response = yield call(fetch, `${SZTM_API}/api/sztm/players?${queryParams}`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'x-access-token': token
      }
    })
    if (response.status != 200) {
      throw Error(response.status)
    } else {
      const responseJson = yield response.json()
      console.log(responseJson)
      yield put(actions.matchesGetPlayersSuccess(responseJson.players))
      yield put(actions.matchesSetFilter({}))
    }
  } catch (error) {
    console.log('Players failure!', actions.matchesGetPlayersFailure(error))
    yield put(actions.matchesGetPlayersFailure(error))
  }
}

function * setFilter (action) {
  const state = yield select()
  const { matches, players, filter } = state.matches
  const filteredMatches = matches.filter(match => {
    return (
      (filter.category ? match.category.includes(filter.category) : true) && 
      (filter.place ? match.place.includes(filter.place) : true) && 
      (filter.date ? moment(match.date).format('DD.MM.YYYY').includes(filter.date) : true) &&
      (filter.time ? moment(match.date).format('HH:mm').includes(filter.time) : true) &&
      (filter.player ? 
        (match.player1 && match.player1.fullName.includes(filter.player)) || (match.player2 && match.player2.fullName.includes(filter.player)) :
        true)
    )
  })

  const filteredPlayers = players.filter(player => {
    return (
      (filter.player ? player.fullName.includes(filter.player) : true) &&
      (filter.category ? player.category.includes(filter.category) : true) &&
      (filter.junior ? player.junior : true) &&
      (filter.paid ? player.paid : true)
    )
  })
  
  yield put(actions.matchesFilterApplied(filteredMatches, filteredPlayers))
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Config saga started.')
  yield takeEvery('MATCHES/GET_MATCHES_REQUEST', getMatches)
  yield takeEvery('MATCHES/GET_PLAYERS_REQUEST', getPlayers)
  yield takeEvery('MATCHES/SET_FILTER', setFilter)
}
